

# Classe Sprite 
Sprite->Get('0')->place(61,299);
Sprite->Get('sex')->place(1030,557);
Sprite->Get('items0')->place(359,392);
Sprite->Get('items1')->place(360,507);
Sprite->Get('items2')->place(361,452);
Sprite->Get('items3')->place(361,601);
Sprite->Get('titre')->place(377,59);


# Classe Diag 


# Classe Link 


# Classe Arrow 


# Classe TW 
#-------------
