Sprite->Get('desc1-to')->place(1425,850);
Sprite->Get('desc1')->place(1307,776);
Sprite->Get('desc2-to')->place(933,1032);
Sprite->Get('desc2')->place(761,1042);
Sprite->Get('items0')->place(56,133);
Sprite->Get('items1')->place(49,388);
Sprite->Get('items2')->place(53,530);
Sprite->Get('titre')->place(351,49);
Sprite->Get('ex')->place(1016,564);
Sprite->Get('ex1')->place(844,861);
Sprite->Get('ex2')->place(950,1023);
Sprite->Get('desc0')->place(775,662);
Sprite->Get('desc0-to')->place(999,632);
Link->Get('desc0-desc0-to')->ftpos(3,7);
Link->Get('desc1-desc1-to')->ftpos(5,0);
Link->Get('desc2-desc2-to')->ftpos(3,7);
