Tk::SlideShow::Sprite->Get('a0')->place(114,156);
Tk::SlideShow::Sprite->Get('b0')->place(482,227);
Tk::SlideShow::Sprite->Get('a1')->place(108,304);
Tk::SlideShow::Sprite->Get('title')->place(287,61);
Tk::SlideShow::Sprite->Get('b1')->place(228,382);
Tk::SlideShow::Sprite->Get('a2')->place(116,458);
Tk::SlideShow::Sprite->Get('b2')->place(251,529);
