Sprite->Get('notez0')->place(1203,521);
Sprite->Get('desc1-to')->place(816,968);
Sprite->Get('desc1')->place(636,923);
Sprite->Get('gus1')->place(710,597);
Sprite->Get('notez1')->place(1218,693);
Sprite->Get('desc2-to')->place(819,998);
Sprite->Get('desc2')->place(637,996);
Sprite->Get('gus2')->place(854,530);
Sprite->Get('notez2')->place(1210,1015);
Sprite->Get('desc3-to')->place(822,1026);
Sprite->Get('desc3')->place(634,1065);
Sprite->Get('gus3')->place(980,590);
Sprite->Get('titre')->place(421,52);
Sprite->Get('oli.gif')->place(642,340);
Sprite->Get('ex')->place(831,847);
Sprite->Get('desc0-to')->place(818,896);
Sprite->Get('desc0')->place(692,817);
Link->Get('gus2-gus3')->ftpos(3,0);
Link->Get('desc3-desc3-to')->ftpos(3,7);
Link->Get('desc0-desc0-to')->ftpos(5,7);
Link->Get('desc1-desc1-to')->ftpos(3,0);
Link->Get('gus1-gus2')->ftpos(2,7);
Link->Get('desc2-desc2-to')->ftpos(3,7);
Arrow->Get('gus3-oli.gif')->ftpos(4,7)->width(8)->shape(23,37,21);
