

# Classe Sprite 
Sprite->Get('desc1-to')->place(905,815);
Sprite->Get('desc1')->place(1138,637);
Sprite->Get('desc2-to')->place(679,859);
Sprite->Get('desc2')->place(611,746);
Sprite->Get('desc3-to')->place(738,897);
Sprite->Get('desc3')->place(603,918);
Sprite->Get('desc4-to')->place(832,936);
Sprite->Get('desc4')->place(631,1062);
Sprite->Get('desc5-to')->place(904,991);
Sprite->Get('desc5')->place(1026,1081);
Sprite->Get('titre')->place(224,101);
Sprite->Get('suite')->place(1331,1107);
Sprite->Get('ex')->place(706,826);
Sprite->Get('desc0')->place(647,649);
Sprite->Get('desc0-to')->place(785,821);


# Classe Diag 


# Classe Link 
Link->Get('desc3-desc3-to')->ftpos(3,7);
Link->Get('desc4-desc4-to')->ftpos(1,6);
Link->Get('desc0-desc0-to')->ftpos(3,1);
Link->Get('desc5-desc5-to')->ftpos(0,4);
Link->Get('desc1-desc1-to')->ftpos(5,2);
Link->Get('desc2-desc2-to')->ftpos(5,0);


# Classe Arrow 


# Classe TW 
#-------------
