Tk::SlideShow::Sprite->Get('a0')->place(142,161);
Tk::SlideShow::Sprite->Get('a1')->place(191,404);
Tk::SlideShow::Sprite->Get('title')->place(238,57);
Tk::SlideShow::Sprite->Get('a2')->place(220,463);
Tk::SlideShow::Sprite->Get('a3')->place(310,535);
Tk::SlideShow::Sprite->Get('a4')->place(363,608);
Tk::SlideShow::Sprite->Get('ex')->place(799,530);
